/**************************************************************/
/* ********************************************************** */
/* *                                                        * */
/* *  Copyright (C) 2001-2011                               * */
/* *  Boris Konev                                           * */
/* *  The University of Liverpool                           * */
/* *                                                        * */
/* *  Copyright (C) 2012-2014                               * */
/* *  Viktor Schuppan                                       * */
/* *                                                        * */
/* *  This program is free software; you can redistribute   * */
/* *  it and/or modify it under the terms of the GNU        * */
/* *  General Public License as published by the Free       * */
/* *  Software Foundation; either version 2 of the License, * */
/* *  or (at your option) any later version.                * */
/* *                                                        * */
/* *  This program is distributed in the hope that it will  * */
/* *  be useful, but WITHOUT ANY WARRANTY; without even     * */
/* *  the implied warranty of MERCHANTABILITY or FITNESS    * */
/* *  FOR A PARTICULAR PURPOSE.  See the GNU General Public * */
/* *  License for more details.                             * */
/* *                                                        * */
/* *  You should have received a copy of the GNU General    * */
/* *  Public License along with this program; if not, write * */
/* *  to the Free Software Foundation, Inc., 59 Temple      * */
/* *  Place, Suite 330, Boston, MA  02111-1307  USA         * */
/* *                                                        * */
/* *                                                        * */
/* ********************************************************** */
/**************************************************************/
#ifndef __OPTIONS_H__
#define __OPTIONS_H__

#include <iostream>
#include <string>

#include "genoptions.h"
#include "misc/exception.h"

namespace Exceptions
{
    class OptionException : public Exception
    {
    public:
        virtual void 
        module(std::ostream& os) const
        {
            os << "Program options: ";
        }
    }; // class ClauseSetException
    class WrongOption : public OptionException
    {
    private:
        const std::string myOption;
    public:
        WrongOption(const std::string& option)
        : myOption (option)
        { }
        virtual void 
        what (std::ostream& os) const
        {
            os << myOption << std::endl;
        }
    }; //class IndexInconsistent
}

typedef enum { INPUT_FORMAT_SNF,
               INPUT_FORMAT_LTL } InputFormat;

typedef enum { EXTRACT_UNSAT_CORE_FORMAT_INTERNAL,
               EXTRACT_UNSAT_CORE_FORMAT_SNF,
               EXTRACT_UNSAT_CORE_FORMAT_LTLC,
               EXTRACT_UNSAT_CORE_FORMAT_LTL } ExtractUnsatCoreFormat;

typedef enum { EXTRACT_UNSAT_CORE_ALGORITHM_PROOF,
               EXTRACT_UNSAT_CORE_ALGORITHM_DELETION,
               EXTRACT_UNSAT_CORE_ALGORITHM_PROOFDELETION }
  ExtractUnsatCoreAlgorithm;

typedef enum { EXTRACT_UNSAT_CORE_PARIKH_ALGORITHM_GAWRYCHOWSKI,
               EXTRACT_UNSAT_CORE_PARIKH_ALGORITHM_SAWA }
  ExtractUnsatCoreParikhAlgorithm;

typedef enum { SHARING_NONE,
               SHARING_SAMEPOLARITY } Sharing;

typedef enum { EXTRACT_UNSAT_CORE_FORMAT_RG_VERTEX_LABEL_INTERNAL,
               EXTRACT_UNSAT_CORE_FORMAT_RG_VERTEX_LABEL_SNF }
  ExtractUnsatCoreFormatRGVertexLabel;

class
TRPPPOptions
{
public:
    TRPPPOptions(int argc, char** argv);

    void
    cleanup();

    const std::string&
    getFileName() const  { return myFileName; }

    const std::string&
    getOrderFileName() const  { return myOrderFile; }

    InputFormat
    getInputFormat() const { return myInputFormat; }

    bool
    isShowInput() const  { return myShowInput; }

    bool
    isShowResult() const { return myShowResult; }

    bool 
    isQuiet() const      { return myQuiet; }

    bool
    isUsePretest() const { return myUsePretest; }

    bool
    isDFS() const        { return myDFS; }

    bool
    isUseFSR() const { return myUseFSR; }

    bool
    isExtractUnsatCore() const
    { return myExtractUnsatCore; }

    void
    setExtractUnsatCore(bool extractUnsatCore)
    { myExtractUnsatCore = extractUnsatCore; }

    const std::string &
    getExtractUnsatCoreFileName() const { return myExtractUnsatCoreFileName; }

    ExtractUnsatCoreFormat
    getExtractUnsatCoreFormat() const { return myExtractUnsatCoreFormat; }

    ExtractUnsatCoreAlgorithm
    getExtractUnsatCoreAlgorithm() const { return myExtractUnsatCoreAlgorithm; }

    bool
    isExtractUnsatCoreSetsOfTimePoints() const
    { return myExtractUnsatCoreSetsOfTimePoints; }

    void
    setExtractUnsatCoreSetsOfTimePoints(bool extractUnsatCoreSetsOfTimePoints)
    { myExtractUnsatCoreSetsOfTimePoints = extractUnsatCoreSetsOfTimePoints; }

    ExtractUnsatCoreParikhAlgorithm
    getExtractUnsatCoreParikhAlgorithm() const
    { return myExtractUnsatCoreParikhAlgorithm; }

    bool
    isExtractUnsatCorePartitionedPropositions() const
    { return myExtractUnsatCorePartitionedPropositions; }

    Sharing
    getSharing() const { return mySharing; }

    bool
    isNoNnf() const { return myNoNnf; }

    bool
    isNoLTLSimplification() const { return myNoLTLSimplification; }

    bool
    isNoDirectClauses() const { return myNoDirectClauses; }

    bool
    isNoEclauseReduction() const { return myNoEclauseReduction; }

    bool
    isNoSorting() const { return myNoSorting; }

    bool
    isNoSNFSimplification() const { return myNoSNFSimplification; }

    bool
    isNoUnsatCoreSimplification() const { return myNoUnsatCoreSimplification; }

    bool
    isNoMakeShared() const { return myNoMakeShared; }

    bool
    isNoRGNoEdgeAug2() const { return myNoRGNoEdgeAug2; }

    bool
    isNoRGNoEdgeBFSLoopItInitCG() const { return myNoRGNoEdgeBFSLoopItInitCG; }

    bool
    isNoRGNoEdgeBFSLoopItInitCE() const { return myNoRGNoEdgeBFSLoopItInitCE; }

    bool
    isNoRGNoEdgeBFSLoopConclusion2E() const
    { return myNoRGNoEdgeBFSLoopConclusion2E; }

    bool
    isNoPruneMainFwdReachable() const { return myNoPruneMainFwdReachable; }

    bool
    isNoPruneMainBwdReachable() const { return myNoPruneMainBwdReachable; }

    bool
    isNoPruneFailedLoopSearchIts() const { return myNoPruneFailedLoopSearchIts; }

    const std::string &
    getExtractUnsatCoreWriteRGBeforePruningFileName() const
    { return myExtractUnsatCoreWriteRGBeforePruningFileName; }

    const std::string &
    getExtractUnsatCoreWriteRGAfterPruningFileName() const
    { return myExtractUnsatCoreWriteRGAfterPruningFileName; }

    ExtractUnsatCoreFormatRGVertexLabel
    getExtractUnsatCoreFormatRGVertexLabel() const
    { return myExtractUnsatCoreFormatRGVertexLabel; }

    bool
    isNuSMVSyntaxUnsatCore() const { return myNuSMVSyntaxUnsatCore; }

private:
    gengetopt_args_info
    myArgsInfo; 

    void
    reportFailure(std::string s = "");

    std::string
    myFileName;

    std::string 
    myOrderFile;

    InputFormat
    myInputFormat;

    bool 
    myShowInput;

    bool
    myShowResult;

    bool 
    myQuiet;

    bool 
    myDFS;

    bool
    myUsePretest;

    bool
    myUseFSR;

    bool
    myExtractUnsatCore;

    std::string
    myExtractUnsatCoreFileName;

    ExtractUnsatCoreFormat
    myExtractUnsatCoreFormat;

    ExtractUnsatCoreAlgorithm
    myExtractUnsatCoreAlgorithm;

    bool
    myExtractUnsatCoreSetsOfTimePoints;

    ExtractUnsatCoreParikhAlgorithm
    myExtractUnsatCoreParikhAlgorithm;

    bool
    myExtractUnsatCorePartitionedPropositions;

    Sharing
    mySharing;

    bool
    myNoNnf;

    bool
    myNoLTLSimplification;

    bool
    myNoDirectClauses;

    bool
    myNoEclauseReduction;

    bool
    myNoSorting;

    bool
    myNoSNFSimplification;

    bool
    myNoUnsatCoreSimplification;

    bool
    myNoMakeShared;

    bool
    myNoRGNoEdgeAug2;

    bool
    myNoRGNoEdgeBFSLoopItInitCG;

    bool
    myNoRGNoEdgeBFSLoopItInitCE;

    bool
    myNoRGNoEdgeBFSLoopConclusion2E;

    bool
    myNoPruneMainFwdReachable;

    bool
    myNoPruneMainBwdReachable;

    bool
    myNoPruneFailedLoopSearchIts;

    std::string
    myExtractUnsatCoreWriteRGBeforePruningFileName;

    std::string
    myExtractUnsatCoreWriteRGAfterPruningFileName;

    ExtractUnsatCoreFormatRGVertexLabel
    myExtractUnsatCoreFormatRGVertexLabel;

    bool
    myNuSMVSyntaxUnsatCore;
};
#endif //  __OPTIONS_H__
